﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.ServiceBus.Description;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Messaging;
using Microsoft.ServiceBus.Messaging.Filters;

namespace NetTopicSample
{
    class Program
    {
        static void Main(string[] args)
        {
            /* Note, to run these samples, you need an Azure Service Bus account,
             * an active Azure Service Bus namespace and an ACS V2 namespace.
             * Please go to http://Azure.com for more information on getting started.
             */ 

            string issuer = "owner";
            string acsHostName = "Your ACS Namespace";
            string topicName = "OrdersTopic";
            string key = "Your Key";
            string serviceNamespace = "Your Namespace";


            /*********************************
             *** Administrative Operations ***
             *********************************/

            // Configure and create NamespaceManager for performing administrative operations
            NamespaceManagerSettings settings = new NamespaceManagerSettings();
            settings.TokenProvider = TokenProvider.CreateSharedSecretTokenProvider(issuer, key);
            NamespaceManager manager = new NamespaceManager(ServiceBusEnvironment.CreateServiceUri("sb", serviceNamespace,string.Empty), settings);


            if (!manager.TopicExists(topicName))
            {
                TopicDescription topic = manager.CreateTopic(topicName);
            }

            // Create Subscriptions for InventoryServiceSubscription and CreditServiceSubscription and associate to OrdersTopic:
            SubscriptionDescription inventoryServiceSubscription = new SubscriptionDescription(topicName,"InventoryServiceSubscription");
            SubscriptionDescription creditServiceSubscription = new SubscriptionDescription(topicName, "CreditServiceSubscription");


            // Set up Filters for NorthAmericaFulfillmentServiceSubscription
            RuleDescription northAmericafulfillmentRuleDescription = new RuleDescription();
            northAmericafulfillmentRuleDescription.Filter= new SqlFilter("CountryOfOrigin = 'USA' OR CountryOfOrigin ='Canada' OR CountryOfOrgin ='Mexico'");
            northAmericafulfillmentRuleDescription.Action =new SqlRuleAction("set FulfillmentRegion='North America'");

            // Set up Filters for AsiaFulfillmentServiceSubscription
            RuleDescription asiaFulfillmentRuleDescription = new RuleDescription();
            asiaFulfillmentRuleDescription.Filter = new SqlFilter("CountryOfOrigin = 'Japan' OR CountryOfOrigin = 'South Korea' OR CountryOfOrigin = 'Thailand'");
            asiaFulfillmentRuleDescription.Action = new SqlRuleAction("set FulfillmentRegion='Asia'");

            // Create Subscriptions for NorthAmericaFulfillmentServiceSubscription and AsiaFulfillmentServiceSubscription and associate to OrdersTopic:
            SubscriptionDescription northAmericaFulfillmentServiceSubscription = new SubscriptionDescription(topicName, "NorthAmericaFulfillmentServiceSubscription");
            SubscriptionDescription asiaFulfillmentServiceSubscription = new SubscriptionDescription(topicName, "AsiaFulfillmentServiceSubscription");

            // Add Subscriptions and Rules to Topic
            if (!manager.SubscriptionExists(topicName, inventoryServiceSubscription.Name))
            {
                manager.CreateSubscription(inventoryServiceSubscription);
            }
            if (!manager.SubscriptionExists(topicName, creditServiceSubscription.Name))
            {
                manager.CreateSubscription(creditServiceSubscription);
            }
            if (!manager.SubscriptionExists(topicName, northAmericaFulfillmentServiceSubscription.Name))
            {
                manager.CreateSubscription(northAmericaFulfillmentServiceSubscription, northAmericafulfillmentRuleDescription);
            }
            if (!manager.SubscriptionExists(topicName, asiaFulfillmentServiceSubscription.Name))
            {
                manager.CreateSubscription(asiaFulfillmentServiceSubscription, asiaFulfillmentRuleDescription);
            }
            

            /**************************
             *** Runtime Operations ***
             **************************/

            // Create and Configure Messaging Factory to provision TopicClient
            MessagingFactorySettings runtimeSettings = new MessagingFactorySettings();
            runtimeSettings.TokenProvider = TokenProvider.CreateSharedSecretTokenProvider(issuer,key);
            MessagingFactory messagingFactory =MessagingFactory.Create(ServiceBusEnvironment.CreateServiceUri("sb", serviceNamespace, String.Empty),runtimeSettings);

            // Create Topic Client for sending messages to the Topic:
            TopicClient client = messagingFactory.CreateTopicClient(topicName);

            /***********************
             *** Send Operations ***
             ***********************/

            // Prepare BrokeredMessage and corresponding properties
            Order order = new Order();
            order.OrderId = 42;
            order.Products.Add("Kinect", 70.50M);
            order.Products.Add("XBOX 360", 199.99M);
            order.Total = order.Products["Kinect"] + order.Products["XBOX 360"];

            // Set the body to the Order data contract
            BrokeredMessage msg = new BrokeredMessage(order);

            // Set properties for use in RuleDescription
            msg.Properties.Add("CountryOfOrigin", "Japan");
            msg.Properties.Add("FulfillmentRegion", "");

            // Send the message to the OrdersTopic
            client.Send(msg);
            client.Close();

            /**************************
              *** Receive Operations ***
            ****************************/

            BrokeredMessage recdMsg;
            Order recdOrder;

            // Inventory Service Subscriber
            SubscriptionClient inventoryServiceSubscriber = messagingFactory.CreateSubscriptionClient(topicName, "InventoryServiceSubscription", ReceiveMode.PeekLock);

            // Read the message from the OrdersTopic
            while ((recdMsg = inventoryServiceSubscriber.Receive(TimeSpan.FromSeconds(5))) != null)
            {
                // Convert from BrokeredMessage to native Order
                recdOrder = recdMsg.GetBody<Order>();

                // Complete read, release and delete message from the fabric
                inventoryServiceSubscriber.Complete(recdMsg.LockToken);

                Console.ForegroundColor = ConsoleColor.Green;
                Console.WriteLine("Received Order {0} \n\t on {1} \n\t with Message Id {2} \n\t and Lock Token {3}.", recdOrder.OrderId, "Inventory Service Subscriber", recdMsg.MessageId, recdMsg.LockToken);
            }
            inventoryServiceSubscriber.Close();

            // Credit Service Subscriber
            SubscriptionClient creditServiceSubscriber = messagingFactory.CreateSubscriptionClient(topicName, "CreditServiceSubscription");

            // Read the message from the OrdersTopic
            recdMsg = creditServiceSubscriber.Receive();

            // Convert from BrokeredMessage to native Order
            recdOrder = recdMsg.GetBody<Order>();

            // Complete read, release and delete message from the fabric
            creditServiceSubscriber.Complete(recdMsg.LockToken);

            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("Received Order {0} \n\t on {1} \n\t with Message Id {2} \n\t and Lock Token {3}.", recdOrder.OrderId, "Credit Service Subscriber", recdMsg.MessageId, recdMsg.LockToken);

            creditServiceSubscriber.Close();

            // Fulfillment Service Subscriber for the North America Fulfillment Service Subscription
            SubscriptionClient northAmericaFulfillmentServiceSubscriber = messagingFactory.CreateSubscriptionClient(topicName, "northAmericaFulfillmentServiceSubscription");

            // Read the message from the OrdersTopic for the North America Fulfillment Service Subscription
            recdMsg = northAmericaFulfillmentServiceSubscriber.Receive(TimeSpan.FromSeconds(5));


            if (recdMsg != null)
            {
                // Convert from BrokeredMessage to native Order
                recdOrder = recdMsg.GetBody<Order>();

                // Complete read, release and delete message from the fabric
                northAmericaFulfillmentServiceSubscriber.Complete(recdMsg.LockToken);

                Console.ForegroundColor = ConsoleColor.Green;
                Console.WriteLine("Received Order {0} \n\t on {1} \n\t with Message Id {2} \n\t and Lock Token {3}.", recdOrder.OrderId, "North America Fulfillment Service Subscriber", recdMsg.MessageId, recdMsg.LockToken);
            }
            else
            {
                Console.ForegroundColor = ConsoleColor.Yellow;
                Console.WriteLine("No messages for North America found.");
            }

            northAmericaFulfillmentServiceSubscriber.Close();

            // Fulfillment Service Subscriber for the Asia Fulfillment Service Subscription
            SubscriptionClient asiaFulfillmentServiceSubscriber = messagingFactory.CreateSubscriptionClient(topicName, "AsiaFulfillmentServiceSubscription");

            // Read the message from the OrdersTopic for the Asia Fulfillment Service Subscription
            recdMsg = asiaFulfillmentServiceSubscriber.Receive(TimeSpan.FromSeconds(5));

            if (recdMsg != null)
            {
                // Convert from BrokeredMessage to native Order
                recdOrder = recdMsg.GetBody<Order>();

                // Complete read, release and delete message from the fabric
                asiaFulfillmentServiceSubscriber.Complete(recdMsg.LockToken);

                Console.ForegroundColor = ConsoleColor.Green;
                Console.WriteLine("Received Order {0} \n\t on {1} \n\t with Message Id {2} \n\t and Lock Token {3}.", recdOrder.OrderId, "Asia Fulfillment Service Subscriber", recdMsg.MessageId, recdMsg.LockToken);

            }
            else
            {
                Console.ForegroundColor = ConsoleColor.Yellow;
                Console.WriteLine("No messages for Asia found.");
            }

            asiaFulfillmentServiceSubscriber.Close();

            Console.ForegroundColor = ConsoleColor.White;
            Console.WriteLine("Hit any key to exit.");
            Console.ReadLine();

        }
    }
}
