﻿//---------------------------------------------------------------------------------
// Microsoft (R)  Windows Azure AppFabric SDK
// Software Development Kit
// 
// Copyright (c) Microsoft Corporation. All rights reserved.  
//
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. 
//---------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Threading;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Description;
using Microsoft.ServiceBus.Messaging;
using Microsoft.ServiceBus.Messaging.Filters;

namespace Microsoft.Samples.PublishSubscribe
{
    public class Program
    {

        #region Fields
        private static DataTable issues;
        private static List<BrokeredMessage> MessageList;
        private static string ServiceNamespace;
        private static string IssuerName;
        private static string IssuerKey;
        #endregion

        public static void Start(string[] args)
        {
            // Set console width
            Console.WindowWidth = 100;
            Console.BufferWidth = 100;

            // Collect user input
            CollectUserInput();

            // Populate test data
            issues = ParseCSVFile();
            MessageList = GenerateMessages(issues);

            // Use Queues to send and receive messages
            Queue();

            // Use Topic and Subscription to send and receive messages
            Topic();
            
            // User Rules with Filters and Actions
            Rules();
            
            // Use Sessions for grouping messages
            Sessions();
        }

        static void Queue()
        {
            //*****************************************************************************************************
            //                                     Management Operations
            //*****************************************************************************************************
            Console.WriteLine("Welcome to the Queue demo. Here we will show a simple temporal decoupling pattern using a Queue. Press ENTER to start.");
            Console.ReadLine();


            // Create management credentials
            SharedSecretCredential myManagementCredentials = TransportClientCredentialBase.CreateSharedSecretCredential(IssuerName, IssuerKey);

            // Create namespace client
            ServiceBusNamespaceClient namespaceClient = new ServiceBusNamespaceClient(ServiceBusEnvironment.CreateServiceUri("sb", ServiceNamespace, string.Empty), myManagementCredentials);

            // Create a queue
            Console.WriteLine("Creating Queue...");
            
            // Delete if exists
            try
            {
                namespaceClient.DeleteQueue("IssueTrackingQueue");
            }
            catch (MessagingEntityNotFoundException)
            { }

            Queue myQueue;
            myQueue = namespaceClient.CreateQueue("IssueTrackingQueue");

            //*****************************************************************************************************
            //                                   Runtime Operations
            //*****************************************************************************************************

            // Create the MessagingFactory
            MessagingFactory factory = MessagingFactory.Create(ServiceBusEnvironment.CreateServiceUri("sb", ServiceNamespace, string.Empty),
                TransportClientCredentialBase.CreateSharedSecretCredential(IssuerName, IssuerKey));

            // Create queue client
            QueueClient myQueueClient = factory.CreateQueueClient(myQueue);

            //*****************************************************************************************************
            //                                   Sending messages to a Queue
            //*****************************************************************************************************

            // Create a sender
            MessageSender myMessageSender = myQueueClient.CreateSender();

            // Send messages
            Console.WriteLine("Now sending messages to the Queue.");
            for (int count = 0; count < 6; count++)
            {
                var issue = MessageList[count];
                issue.Label = issue.Properties["IssueTitle"].ToString();
                myMessageSender.Send(issue);
                Console.WriteLine(string.Format("Message sent: {0}, {1}", issue.Label, issue.MessageId));
            }

            //*****************************************************************************************************
            //                                   Receiving messages from a Queue
            //*****************************************************************************************************

            // Create a receiver
            MessageReceiver myMessageReceiver = myQueueClient.CreateReceiver(ReceiveMode.PeekLock);

            // Receive messages
            Console.WriteLine("Now receiving messages from Queue.");
            BrokeredMessage message;
            while (myMessageReceiver.TryReceive(new TimeSpan(hours: 0, minutes: 0, seconds: 5), out message))
            {
                Console.WriteLine(string.Format("Message received: {0}, {1}, {2}", message.SequenceNumber, message.Label, message.MessageId));
                message.Complete();

                Console.WriteLine("Processing message (sleeping...)");
                Thread.Sleep(1000);
            }

            Console.WriteLine("End of scenario, press ENTER to clean up.");
            Console.ReadLine();

            // cleanup
            myMessageReceiver.Close();
            myMessageSender.Close();
            myQueueClient.Close();
            namespaceClient.DeleteQueue("IssueTrackingQueue");
        }

        static void Topic()
        {
            //*****************************************************************************************************
            //                                     Management Operations
            //*****************************************************************************************************
            Console.WriteLine("Welcome to the PubSub  demo. Here we will send messages to a Topic with two subscrptions. Press ENTER to start.");
            Console.ReadLine();

            // Create management credentials
            SharedSecretCredential myManagementCredentials = TransportClientCredentialBase.CreateSharedSecretCredential(IssuerName, IssuerKey);

            // Create namespace client
            ServiceBusNamespaceClient namespaceClient = new ServiceBusNamespaceClient(ServiceBusEnvironment.CreateServiceUri("sb", ServiceNamespace, string.Empty), myManagementCredentials);

            // Create a topic
            Console.WriteLine("Creating Topic...");

            // Delete if exists
            try
            {
                namespaceClient.DeleteTopic("IssueTrackingTopic");
            }
            catch (MessagingEntityNotFoundException)
            { }

            Topic myTopic;
            myTopic = namespaceClient.CreateTopic("IssueTrackingTopic");

            // Create a subscription
            Console.WriteLine("Creating Subscriptions...");
            Subscription myAuditSubscription = myTopic.AddSubscription("AuditSubscription");
            Subscription myAgentSubscription = myTopic.AddSubscription("AgentSubscription");

            //*****************************************************************************************************
            //                                   Runtime Operations
            //*****************************************************************************************************

            // Create the MessagingFactory
            MessagingFactory factory = MessagingFactory.Create(ServiceBusEnvironment.CreateServiceUri("sb", ServiceNamespace, string.Empty),
                TransportClientCredentialBase.CreateSharedSecretCredential(IssuerName, IssuerKey));

            //*****************************************************************************************************
            //                                   Sending messages to a Topic
            //*****************************************************************************************************

            // Create topic client
            TopicClient myTopicClient = factory.CreateTopicClient(myTopic);

            // Create a sender
            MessageSender myMessageSender = myTopicClient.CreateSender();

            // Send messages
            Console.WriteLine("Now sending messages to Topic.");
            foreach (var issue in MessageList)
            {
                issue.Label = issue.Properties["IssueTitle"].ToString();
                myMessageSender.Send(issue);
                Console.WriteLine(string.Format("Message sent: {0}, {1}", issue.Label, issue.MessageId));
            }

            //*****************************************************************************************************
            //                                   Receiving messages from a Subscription
            //*****************************************************************************************************

            // Create subscription client
            SubscriptionClient mySubscriptionClient = factory.CreateSubscriptionClient(myAgentSubscription);

            // Create a receiver using PeekLock mode
            MessageReceiver myMessageReceiver = mySubscriptionClient.CreateReceiver(ReceiveMode.PeekLock);

            // Receive messages
            Console.WriteLine("Now receiving messages from AgentSubscription..");
            BrokeredMessage message;
            while (myMessageReceiver.TryReceive(new TimeSpan(hours: 0, minutes: 0, seconds: 5), out message))
            {
                Console.WriteLine(string.Format("Message received (AgentSubscription): {0}, {1}, {2}", message.SequenceNumber, message.Label, message.MessageId));
                message.Complete();
            }

            // Create subscription client
            mySubscriptionClient = factory.CreateSubscriptionClient(myAuditSubscription);

            // Create a receiver using ReceiveAndDelete mode
            myMessageReceiver = mySubscriptionClient.CreateReceiver(ReceiveMode.ReceiveAndDelete);

            // Receive messages
            Console.WriteLine("Now receiving messages from AuditSubscription..");
            while (myMessageReceiver.TryReceive(new TimeSpan(hours: 0, minutes: 0, seconds: 5), out message))
            {
                Console.WriteLine(string.Format("Message received (AuditSubscription): {0}, {1}, {2}", message.SequenceNumber, message.Label, message.MessageId));
            }

            Console.WriteLine("End of scenario, press ENTER to clean up.");
            Console.ReadLine();

            // cleanup
            myMessageReceiver.Close();
            myMessageSender.Close();
            myTopicClient.Close();
            mySubscriptionClient.Close();
            namespaceClient.DeleteTopic("IssueTrackingTopic");
        }

        static void Rules()
        {
            //*****************************************************************************************************
            //                                     Management Operations
            //*****************************************************************************************************

            Console.WriteLine("Welcome to the Topic / Rules demo. Here we will stamp messages with new properties based on their content. Press ENTER to start.");
            Console.ReadLine();

            // Create management credentials
            SharedSecretCredential myManagementCredentials = TransportClientCredentialBase.CreateSharedSecretCredential(IssuerName, IssuerKey);

            // Create namespace client
            ServiceBusNamespaceClient namespaceClient = new ServiceBusNamespaceClient(ServiceBusEnvironment.CreateServiceUri("sb", ServiceNamespace, string.Empty), myManagementCredentials);

            // Create a topic
            Console.WriteLine("Creating Topic...");

            // Delete if exists
            try
            {
                namespaceClient.DeleteTopic("IssueTrackingTopic");
            }
            catch (MessagingEntityNotFoundException)
            { }

            Topic myTopic;
            myTopic = namespaceClient.CreateTopic("IssueTrackingTopic");

            // Create a Rule that selectively stamps Priority and Imporance values
            RuleDescription highPriorityRuleDescription = new RuleDescription();
            highPriorityRuleDescription.FilterExpression = new SqlFilterExpression("SupportPackage = 'Premium' OR Severity <= 1");
            highPriorityRuleDescription.FilterAction = new SqlFilterAction("set Priority = 1;set Importance = 'High';");

            // Create a Subscription and apply rule
            Subscription mySubscription = myTopic.AddSubscription("IssueTrackingSubscription", highPriorityRuleDescription);

            //*****************************************************************************************************
            //                                   Runtime Operations
            //*****************************************************************************************************

            // Create the MessagingFactory
            MessagingFactory factory = MessagingFactory.Create(ServiceBusEnvironment.CreateServiceUri("sb", ServiceNamespace, string.Empty),
                TransportClientCredentialBase.CreateSharedSecretCredential(IssuerName, IssuerKey));

            //*****************************************************************************************************
            //                             Sending messages to Topic
            //*****************************************************************************************************

            // Create topic client
            TopicClient myTopicClient = factory.CreateTopicClient(myTopic);

            // Create a sender
            MessageSender myMessageSender = myTopicClient.CreateSender();

            // Send messages
            Console.WriteLine("Sending messages to Topic.");
            foreach (var issue in MessageList)
            {
                issue.Label = issue.Properties["IssueTitle"].ToString();
                Console.WriteLine(string.Format("Sent values: SupportPackage={0}, Severity={1}, Priority={2}.", issue.Properties["SupportPackage"], issue.Properties["Severity"], issue.Properties["Priority"]));
                myMessageSender.Send(issue);
            }

            //*****************************************************************************************************
            //                       Receiving messages from Subscription
            //*****************************************************************************************************

            // Create subscription client
            SubscriptionClient mySubscriptionClient = factory.CreateSubscriptionClient(mySubscription);

            // Create a receiver
            MessageReceiver myMessageReceiver = mySubscriptionClient.CreateReceiver(ReceiveMode.PeekLock);

            // Receive messages
            Console.WriteLine("Receiving filtered messages with added property value for Importance.");
            BrokeredMessage message;
            while (myMessageReceiver.TryReceive(new TimeSpan(hours: 0, minutes: 0, seconds: 5), out message))
            {
                Console.WriteLine(string.Format("Received values: SupportPackage:{0}, Severity:{1}, Priority:{2}, Importance:{3}", message.Properties["SupportPackage"], message.Properties["Severity"], message.Properties["Priority"], message.Properties["Importance"]));
                message.Complete();
            }

            myMessageReceiver.Close();
            mySubscriptionClient.Close();

            Console.WriteLine("End of scenario, press ENTER to clean up.");
            Console.ReadLine();

            // cleanup
            myMessageReceiver.Close();
            myMessageSender.Close();
            myTopicClient.Close();
            mySubscriptionClient.Close();
            namespaceClient.DeleteTopic("IssueTrackingTopic");
        }

        static void Sessions()
        {
            //*****************************************************************************************************
            //                                     Management Operations
            //*****************************************************************************************************

            Console.WriteLine("Welcome to the Topic / Sessions demo. Press ENTER to start.");
            Console.ReadLine();

            // Create management credentials
            SharedSecretCredential myManagementCredentials = TransportClientCredentialBase.CreateSharedSecretCredential(IssuerName, IssuerKey);

            // Create namespace client
            ServiceBusNamespaceClient namespaceClient = new ServiceBusNamespaceClient(ServiceBusEnvironment.CreateServiceUri("sb", ServiceNamespace, string.Empty), myManagementCredentials);

            // Create a topic
            Console.WriteLine("Creating Topic...");

            // Delete if exists
            try
            {
                namespaceClient.DeleteTopic("IssueTrackingTopic");
            }
            catch (MessagingEntityNotFoundException)
            { }

            Topic myTopic;
            myTopic = namespaceClient.CreateTopic("IssueTrackingTopic");


            // Create a subscription
            SubscriptionDescription mySubscriptionDescription = new SubscriptionDescription();
            mySubscriptionDescription.RequiresSession = true;
            Subscription mySubscription = myTopic.AddSubscription("AuditSubscription", mySubscriptionDescription);


            //*****************************************************************************************************
            //                                   Runtime Operations
            //*****************************************************************************************************

            // Create the MessagingFactory
            MessagingFactory factory = MessagingFactory.Create(ServiceBusEnvironment.CreateServiceUri("sb", ServiceNamespace, string.Empty),
                TransportClientCredentialBase.CreateSharedSecretCredential(IssuerName, IssuerKey));

            //*****************************************************************************************************
            //                             Sending messages to a particular session
            //*****************************************************************************************************

            // Create topic client
            TopicClient myTopicClient = factory.CreateTopicClient(myTopic);

            // Create a sender
            MessageSender myMessageSender = myTopicClient.CreateSender();

            // Send messages
            Console.WriteLine("Now sending messages to Topic.");
            foreach (var issue in MessageList)
            {
                issue.Label = issue.Properties["IssueTitle"].ToString();
                issue.SessionId = issue.Properties["CategoryID"].ToString();
                Console.WriteLine(string.Format("Issue properties: Title={0}, Category={1}.", issue.Properties["IssueTitle"], issue.Properties["CategoryID"]));
                myMessageSender.Send(issue);
            }

            //*****************************************************************************************************
            //                       Receiving messages from a particular session
            //*****************************************************************************************************

            // Create subscription client
            SubscriptionClient mySubscriptionClient = factory.CreateSubscriptionClient(mySubscription);

            // Create couple Receivers specifying the SessionId
            MessageReceiver category1MessageReciever = mySubscriptionClient.AcceptSessionReceiver("1");
            MessageReceiver category2MessageReciever = mySubscriptionClient.AcceptSessionReceiver("2");

            // Recieve messages
            Console.WriteLine("Now recieving messages from Subscription.");
            ConsoleColor currentConsoleForeground = Console.ForegroundColor;
            BrokeredMessage message;

            // For messages with SessionId=1
            while (category1MessageReciever.TryReceive(new TimeSpan(hours: 0, minutes: 0, seconds: 5), out message))
            {
                Console.ForegroundColor = ConsoleColor.DarkGreen;
                Console.WriteLine(string.Format("Message recieved for Category1: {0}", message.Label));
                message.Complete();
            }

            // For messages with SessionId=2
            while (category2MessageReciever.TryReceive(new TimeSpan(hours: 0, minutes: 0, seconds: 5), out message))
            {
                Console.ForegroundColor = ConsoleColor.DarkYellow;
                Console.WriteLine(string.Format("Message recieved for Category2: {0}", message.Label));
                message.Complete();
            }
            Console.ForegroundColor = currentConsoleForeground;

            Console.WriteLine("End of scenario, press ENTER to clean up.");
            Console.ReadLine();

            // cleanup
            category1MessageReciever.Close();
            category2MessageReciever.Close();
            myMessageSender.Close();
            myTopicClient.Close();
            mySubscriptionClient.Close();
            namespaceClient.DeleteTopic("IssueTrackingTopic");
        }

        #region Helper Methods

        static void CollectUserInput()
        {
            // User namespace
            Console.Write("Please provide the service namespace to use: ");
            ServiceNamespace = Console.ReadLine();

            // Issuer name
            Console.Write("Please provide the issuer name to use: ");
            IssuerName = Console.ReadLine();

            // Issuer key
            Console.Write("Please provide the issuer key to use: ");
            IssuerKey = Console.ReadLine();
        }

        static List<BrokeredMessage> GenerateMessages(DataTable issues)
        {
            List<BrokeredMessage> result = new List<BrokeredMessage>();
            foreach (DataRow item in issues.Rows)
            {
                BrokeredMessage message = BrokeredMessage.CreateMessage();
                foreach (DataColumn property in issues.Columns)
                {
                    message.Properties.Add(property.ColumnName, item[property]);
                }
                result.Add(message);
            }
            return result;
        }

        static DataTable ParseCSVFile()
        {
            DataTable tableIssues = new DataTable("Issues");
            string path = @"..\..\Sample_PublishSubscribe\data.csv";
            try
            {
                using (StreamReader readFile = new StreamReader(path))
                {
                    string line;
                    string[] row;

                    // create the columns
                    line = readFile.ReadLine();
                    foreach (string columnTitle in line.Split(','))
                    {
                        tableIssues.Columns.Add(columnTitle);
                    }

                    while ((line = readFile.ReadLine()) != null)
                    {
                        row = line.Split(',');
                        tableIssues.Rows.Add(row);
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Error:" + e.ToString());
            }

            return tableIssues;
        }

        #endregion
    }
}
